use strictures 1;
use Test::More;
use DX::Solver;
use DX::OrderedSet;

my $solver = DX::Solver->new;

{ package My::Server;
  use Moo;
  has name => (is => 'ro', required => 1);

  package My::ShellInstalled;
  use Moo;
  has server => (is => 'ro', required => 1);
  has shell => (is => 'ro', required => 1);
}

$solver->facts->{server} = DX::OrderedSet->new(
  values => [ map My::Server->new(name => $_), qw(one two three four five) ]
);
$solver->facts->{shell_installed} = DX::OrderedSet->new(
  values => [
    (map My::ShellInstalled->new(server => $_, shell => 'bash'),
       qw(one three four)),
    (map My::ShellInstalled->new(server => $_, shell => 'csh'),
       qw(two three five)),
  ],
);

my @r = $solver->query([ 'X' ],
  [ findall => X => S =>
    [ member_of => 'S', \'server' ],
    [ exists => [ qw(Name SI) ] =>
      [ member_of => 'SI', \'shell_installed' ],
      [ prop => 'SI' => \'server' => 'Name' ],
      [ prop => 'S' => \'name' => 'Name' ],
      [ prop => 'SI' => \'shell' => \'bash' ]
    ],
  ]
)->results;

is_deeply(
  [ map $_->name, $r[0]->value_for('X')->all ],
  [ qw(one three four) ]
);

done_testing;
